@@ -63,7 +63,7 @@ class KiwoomOpenApiPlusEntrypoint(KiwoomOpenApiPlusEntrypointMixin, Logging):
 
         self.logger.debug('Testing if client is ready...')
         if not self._client.is_ready(self._client_check_timeout):
-            assert platform.architecture() == ('32bit', 'WindowsPE'), 'Server should run under 32bit environment'
+            assert platform.architecture()[0] == '32bit', 'Server should run under 32bit environment'
             self.logger.debug('Client is not ready, creating a new server')
             self._server_proc = subprocess.Popen(self._server_proc_args)
             assert self._client.is_ready(), 'Failed to create server'
