@@ -4,6 +4,8 @@ from koapy.compat.pyside2.QtWidgets import QWidget
 from koapy.compat.pyside2.QtAxContainer import QAxWidget
 from koapy.compat.pyside2.QtCore import QEvent, Qt
 
+from koapy.compat.pyside2 import PYQT5, PYSIDE2, PythonQtError
+
 from koapy.backend.kiwoom_open_api_plus.core.KiwoomOpenApiPlusDynamicCallable import KiwoomOpenApiPlusDynamicCallable
 from koapy.backend.kiwoom_open_api_plus.core.KiwoomOpenApiPlusSignature import KiwoomOpenApiPlusDispatchSignature, KiwoomOpenApiPlusEventHandlerSignature
 from koapy.backend.kiwoom_open_api_plus.core.KiwoomOpenApiPlusSignalConnector import KiwoomOpenApiPlusSignalConnector
@@ -19,15 +21,18 @@ class KiwoomOpenApiPlusQAxWidget(QWidget, KiwoomOpenApiPlusQAxWidgetMixin, Loggi
     CLSID = '{A1574A0D-6BFA-4BD7-9020-DED88711818D}'
     PROGID = 'KHOPENAPI.KHOpenApiCtrl.1'
 
-    CONTROL_NAME_KWARG_KEY = 'c'
-
     METHOD_NAMES = KiwoomOpenApiPlusDispatchSignature.names()
     EVENT_NAMES = KiwoomOpenApiPlusEventHandlerSignature.names()
 
     def __init__(self, *args, **kwargs):
         assert platform.architecture()[0] == '32bit', 'Contorl object should be created in 32bit environment'
 
-        KiwoomOpenApiPlusQAxWidgetMixin.__init__(self)
+        if PYQT5:
+            self.logger.debug('Using PyQt5 as Qt backend')
+        elif PYSIDE2:
+            self.logger.debug('Using PySide2 as Qt backend')
+        else:
+            raise PythonQtError('No Qt bindings could be found')
 
         super_args = args
         super_kwargs = kwargs
@@ -37,11 +42,13 @@ class KiwoomOpenApiPlusQAxWidget(QWidget, KiwoomOpenApiPlusQAxWidgetMixin, Loggi
         if len(args) > 0 and isinstance(args[0], str):
             super_args = args[1:]
             clsid_or_progid = args[0]
-        elif self.CONTROL_NAME_KWARG_KEY in kwargs:
-            super_kwargs = {k:v for k, v in kwargs if k != self.CONTROL_NAME_KWARG_KEY}
-            clsid_or_progid = kwargs[self.CONTROL_NAME_KWARG_KEY]
+        elif 'c' in kwargs:
+            super_kwargs = {k:v for k, v in kwargs if k != 'c'}
+            clsid_or_progid = kwargs['c']
 
         QWidget.__init__(self, *super_args, **super_kwargs)
+        KiwoomOpenApiPlusQAxWidgetMixin.__init__(self)
+        Logging.__init__(self)
 
         self._ax = QAxWidget(clsid_or_progid, self)
 
