@@ -27,10 +27,6 @@ class LoggingMeta(type):
                 "handlers": ["console"],
             },
         },
-        "root": {
-            "level": "DEBUG",
-            "handlers": ["console"],
-        },
         "incremental": False,
         "disable_existing_loggers": False,
     }
@@ -82,7 +78,9 @@ class LoggingMeta(type):
 
     def get_logger(cls, name=None):
         if name is None:
-            name = 'koapy'
+            name = LoggingMeta.__logger_name(cls)
+            if name == '__main__':
+                name = 'koapy'
         LoggingMeta.__initialize_if_necessary(cls)
         logger = logging.getLogger(name)
         return logger
